<?php

#
# PEAR::Iper_API
#
# Author: Christophe Ruelle
# Version: $Revision: 0.9$
#
# requires 
#           PHP 5.2 or higher
#           CURL http://www.php.net/manual/fr/book.curl.php
#           XML  http://www.php.net/manual/fr/book.xml.php

class Iper_API {

  var $_conf = array(
		     'api_key'    => '',
		     'secret' => '',
		     'api_url'    => 'http://api.ipernity.com/api',
		     'auth_url'   => 'http://www.ipernity.com/apps/authorize',
		     'format'     => 'json'
		     );
  
  var $_curl = array('FOLLOWLOCATION'=>1,
		     'MAXREDIRS'=>1);

  var $_version = '1.0';
  var $_app_name = 'Unknown';

  var $_auth = array('token'=>'','user_id'=>0,'username'=>'','realname'=>'','permissions'=>array());
  var $_timeout = 5;
  var $_error_code = 0;
  var $_error_msg = '';
  var $_json_assoc = true;
  var $_maxfilesize = 94371840; // 90MB

  // ============================================================================
  // CONSTRUCTOR Iper_API
  // ----------------------------------------------------------------------------

  function Iper_API($conf=array())
  {
    $this->setConf($conf);
  }

  // ============================================================================
  // setConf(array) : add configuration values
  // ----------------------------------------------------------------------------

  function setConf($conf)
  {
    $this->_conf=array_merge($this->_conf,$conf);
    $this->_checkConf();
  }

  // ============================================================================
  // setAppName(str) : set up your application name (optional)
  // ----------------------------------------------------------------------------

  function setAppName($name)
  {
    $this->_app_name = $name;
  }

  // ============================================================================
  // request : perform an API call with method and parameters (and optional auth_token)
  // ----------------------------------------------------------------------------
  //
  // return :
  //         JSON : array or object json_decode(response,bool) - bool=$this->_json_assoc, true by default
  //         PHP  : array unserialize(response)
  //         XML  : an array of nodes xml_parse_into_struct
  //         false : request failed. check $this->getErrorCode() & $this->getErrorMessage()
  //
  // uploading methods are accepted, simple add a file=path/of/your/file in hte parameters.

  function request($method, $params=array(),$with_auth_token=false) {

    $this->_error_code = 0;
    $this->_error_msg = '';
    $this->_curl_info = array();
    $this->_data = '';

    // *
    // * Create API Query
    // *

    $params['api_key']=$this->_conf['api_key'];

    // add auth_token ?
    if ( $with_auth_token && $this->_auth['token'] )
    {
      $params['auth_token'] = $this->_auth['token'];
    }

    // would like to upload a file?

    if ( $method=='upload.file' || $method=='upload.replace' )
    {
      if ( is_array($_FILES['file']) && $_FILES['file']['size'] && $_FILES['file']['size'] < $this->_maxfilesize)
      {
	$file = $_FILES['file']['tmp_name'];
      }
      elseif ( filesize($params['file']) && filesize($params['file']) < $this->_maxfilesize )
      {
	$file = $params['file'];
      }
      unset($params['file']);
    }
    else { $file=false; }

    // sign request
    if ($this->_conf['secret'])
    {
      $params['api_sig'] = $this->_signParams($method,$params);
    }

    // *
    // * Send API Query
    // *
    
    $url = $this->_conf['api_url'].'/'.$method.'/'.$this->_conf['format'];

    $curl=curl_init();

    if ( $file )
    {
       curl_setopt($curl,CURLOPT_BINARYTRANSFER,1);
       curl_setopt($curl,CURLOPT_RETURNTRANSFER,1);
       curl_setopt($curl,CURLOPT_INFILESIZE,filesize($file));
       $fp=fopen($file,'rb');
       curl_setopt($curl,CURLOPT_INFILE,$fp);
       $params['file']='@'.$file;
    }

    curl_setopt($curl, CURLOPT_POST, 1);
    curl_setopt($curl, CURLOPT_POSTFIELDS, $params);
    curl_setopt($curl, CURLOPT_URL, $url);
    curl_setopt($curl, CURLOPT_HEADER,0);
    curl_setopt($curl, CURLOPT_RETURNTRANSFER,1);
    curl_setopt($curl, CURLOPT_HTTPHEADER,array($this->_getForwardedFor()));
    curl_setopt($curl, CURLOPT_CONNECTTIMEOUT,$this->_timeout);
    curl_setopt($curl, CURLOPT_USERAGENT,$this->_getUserAgent());

    foreach($this->_curl as $o=>$v) curl_setopt($curl,constant('CURLOPT_'.$o),$v);

    $data=curl_exec($curl);       // Execute la commande
    $info=curl_getinfo($curl);

    if ( curl_errno($curl) )
    {
      $this->_error_msg = curl_error($curl);
      return false;
    }
    else if ( $info['http_code'] != 200 || !sizeof($data))
    {
      $this->_error_msg = 'Bad response from remote server (HTTP status code '.$info['http_code'].')';
      return false;
    }

    curl_close($curl);

    if ( $fp ) { fclose($fp); }

    $this->_data = $data;
    $this->_info = $info;

    // *
    // * Parse Response
    // *

    // JSON

    if ( $this->_conf['format']=='json' )
    {
      return $this->_parseJSONResponse($data);
    }

    // PHP

    elseif ( $this->_conf['format']=='php' )
    {
      return $this->_parsePHPResponse($data);
    }

    // XML

    elseif ( $this->_conf['format']=='xml' )
    {
      return $this->_parseXMLResponse($data);
    }

    else
    {
      $this->_error_code = 0;
      $this->_error_msg  = 'Requested API output format is invalid.';
    }
  }

  // ============================================================================
  // getErrorCode/Message : return the last request error code
  // ----------------------------------------------------------------------------

  function getErrorCode() { return $this->_error_code; }
  function getErrorMessage() { return $this->_error_msg; }

  // ============================================================================
  // getAuthUrl : return an authentication url
  // ----------------------------------------------------------------------------

  function getAuthUrl($perms=array(),$frob='') {

    $params = array();

    $O = array('doc','post','network','profile');
    $R = array('','read','write','delete');
    if ( sizeof($perms) )
    {
      foreach($perms as $o=>$r) 
	if ( in_array($o,$O) && in_array($r,$R) ) { $params['perm_'.$o]=$r; }
    }

    $params['api_key'] = $this->_conf['api_key'];
    
    if ( strlen($frob) ) $params['frob']=$frob;

    $params['api_sig'] = $this->_signParams('',$params);

    return $this->_conf['auth_url'].'?'.$this->_buildQueryString($params);
  }

  // ============================================================================
  // getFrob : ask for a frob
  // ----------------------------------------------------------------------------

  function getFrob() {

    $params = array('api_key'=>$this->_conf['api_key']);
    $o = $this->request('auth.getFrob',$params);

    if ( $o )
    {
      if ( $this->_conf['format']=='json' )
      {
	if ( is_object($o)) { $o = json_decode($this->_data,true);}
	if ( isset($o['auth']['frob']) ) 
	  return $o['auth']['frob'];
      }
      elseif ( $this->_conf['format']=='php' )
      {
	if ( isset($o['auth']['frob']) )
	  return $o['auth']['frob'];
      }
      elseif ( $this->_conf['format']=='xml' )
      {
	foreach($o as $node) {
	  if ( $node['tag']=='FROB' && $node['type']=='complete' )  return $node['value'];
	}
      }
    }
    return false;
  }

  // ============================================================================
  // getToken(str) : exchange a frob to an auth_token
  //
  // when a user was redirected to getAuthUrl() to authenticate you will
  // need to exchange the frob to an auth_token
  // ----------------------------------------------------------------------------

  function getToken($frob) {
    
    $o = $this->request('auth.getToken',array('frob'=>$frob));
    if ( $o!=false )
    {
      $auth = $this->_parseAuthToken($o);
      if ( $auth ) { $this->_auth = $auth; return $auth['token']; }
    }
    return false;
  }

  // ============================================================================
  // checkToken(str) : check a token
  //
  // when a user was redirected to getAuthUrl() to authenticate you will
  // need to exchange the frob to an auth_token
  // ----------------------------------------------------------------------------

  function checkToken($token=null) {
    if ( ! $token ) $token = $this->_auth['token'];
    $o = $this->request('auth.checkToken',array('auth_token'=>$token));
    if ( $o!=false )
    {
      $auth = $this->_parseAuthToken($o);
      if ( $auth ) { $this->_auth = $auth; return true; }
    }
    return false;
  }

  // setAuthentication : use this method if you've saved authentication previously

  function setAuthentication($auth) {
    $this->_auth = $auth;
  }

  function getAuthentication() {
    return ($this->_auth['token'])? $this->_auth['token']:false;
  }

  // unAuthenticate : remove authentication

  function unAuthenticate() {
    $this->_auth = array('token'    => '',
			 'user_id'  => 0,
			 'username' => '',
			 'realname' => '',
			 'permissions'=>array());
  }

  function _parseAuthToken($o) {

    if ( $this->_conf['format']=='json' )
    {
      if (is_object($o)) { $o = json_decode($this->_data,true);}

      if ( isset($o['auth']['token']) ){
	return array('token'    => $o['auth']['token'],
		     'user_id'  => $o['auth']['user']['user_id'],
		     'username' => $o['auth']['user']['username'],
		     'realname' => $o['auth']['user']['realname'],
		     'permissions'=> $o['auth']['permissions']);
	
      }
    }
    elseif ( $this->_conf['format']=='php' )
    {
      if ( isset($o['auth']['token']) )
      {
	return array('token'    => $o['auth']['token'],
		     'user_id'  => $o['auth']['user']['user_id'],
		     'username' => $o['auth']['user']['username'],
		     'realname' => $o['auth']['user']['realname'],
		     'permissions'=> $o['auth']['permissions']);
      }
    }
    elseif ( $this->_conf['format']=='xml' )
    {
      $auth = array();
      foreach($o as $node) {
	if ( $node['tag']=='TOKEN' )
	  $auth['token']=$node['value'];
	elseif ( $node['tag']=='USER' )
	{
	  $auth['user_id']=$node['attributes']['USER_ID'];
	  $auth['username']=$node['attributes']['USERNAME'];
	  $auth['realname']=$node['attributes']['REALNAME'];
	}
	elseif ( $node['tag']=='PERMISSIONS' )
	{
	  $auth['permissions']=$node['attributes'];
	}
      }
      if ( $auth['token'] && $auth['user_id'] ) return $auth;
    }
    return false;
  }

  // ============================================================================
  // PRIVATE _checkConf : perform basic checks on configuration & libraries
  // ----------------------------------------------------------------------------

  function _checkConf() {

    if ( $this->_conf['format']=='json' && !function_exists('json_decode') )
      die('JSON not found. Update PHP to >5.2.0 or PECL json:1.2.0-1.2.1');
    elseif ( $this->_conf['format']=='php' && !function_exists('unserialize') )
      die('PHP unserialize not found. Update PHP to 5.x');
    elseif ( $this->_conf['format']=='xml' && !function_exists('xml_parser_create') )
      die('PHP xml_parser_create not found.');
  }

  // ============================================================================
  // PRIVATE _parse responses
  // ----------------------------------------------------------------------------
  
  function _parseJSONResponse($data) {
    $o = json_decode($data,true);

    if ( is_array($o) && isset($o['api']) && isset($o['api']['status']) )
    {
      if ( $o['api']['status']!='ok' )
      {
	$this->_error_code = $o['api']['code'];
	$this->_error_msg  = $o['api']['message'];
	return false;
      }
      else
      {
	return ($this->_json_assoc)? $o : json_decode($data,false);
      }
    }
    $this->_error_code = 0;
    $this->_error_msg  = 'Could not parse the JSON response.';
    return false;
  }
  
  function _parsePHPResponse($data) {
    $o = unserialize($data);
    if ( is_array($o) && isset($o['api']) && isset($o['api']['status']) )
    {
      if ( $o['api']['status']!='ok' )
      {
	$this->_error_code = $o['api']['code'];
	$this->_error_msg  = $o['api']['message'];
	return false;
      }
      else
      {
	return $o;
      }
    }
    $this->_error_code = 0;
    $this->_error_msg  = 'Could not parse the PHP/Serialize response.';
    return false;
  }

  function _parseXMLResponse($data) {
    $xmp  = xml_parser_create();
    xml_parser_set_option($xmp, XML_OPTION_SKIP_WHITE, 1);
    xml_parser_set_option($xmp, XML_OPTION_TARGET_ENCODING, 'UTF-8');
    
    xml_parse_into_struct($xmp, $data, $o, $index);
    xml_parser_free($xmp);

    if ( is_array($o) && sizeof($o) )
    {
      $s = $o[0];
      
      if ( isset($s['attributes']) && isset($s['attributes']['STATUS']) )
      {
	if ( $s['attributes']['STATUS']!='ok' )
	{
	  $this->_error_code = $s['attributes']['CODE'];
	  $this->_error_msg  = $s['attributes']['MESSAGE'];
	  return false;
	}
	else
	{
	  return $o;
	}
      }
    }
    $this->_error_code = 0;
    $this->_error_msg  = 'Could not parse the XML response.';
    return false;
  }

  // ============================================================================
  // PRIVATE _signParams(str,array) : return the parameters api_sig value
  // ----------------------------------------------------------------------------

  function _signParams($method,$params) {
    if ( $this->_conf['secret'] )
    {
      $keys = array_keys($params);
      sort($keys);
      $sig='';
      foreach($keys as $k) { $sig.=$k.$params[$k]; }
      $sig.=$method.$this->_conf['secret'];
      return md5($sig);
    }
  }

  // ============================================================================
  // PRIVATE _buildQueryString
  // ----------------------------------------------------------------------------

  function _buildQueryString($params) {
    $query=array();
    foreach($params as $k=>$v) $query[]=$k.'='.urlencode($v);
    return implode('&',$query);
  }

  // ============================================================================
  // PRIVATE _getRemoteIp : return the calling user IP
  // ----------------------------------------------------------------------------
  
  function _getForwardedFor() {
    return 'X-Forwarded-For: '.($_SERVER['HTTP_CLIENT_IP']?$_SERVER['HTTP_CLIENT_IP'].',':'').($_SERVER['HTTP_X_FORWARDED_FOR']?$_SERVER['HTTP_X_FORWARDED_FOR'].',':'').$_SERVER['REMOTE_ADDR'];
  }

  function _getUserAgent() {
    return 'Iper_API/'.$this->_version.' (PHP/CURL;'.$this->_app_name.';'.$this->_conf['api_key'].')';
  }

}  